rm(list=ls())

## The tips below are useful in shaping up student laptop ratings csv files:
## Regexp and replace on the console:
## find . -name \*.csv -exec sed -i "s/;/,/g" {} \; ## replace every ";" with ","
## find . -name \*.csv -exec sed -i "s/\"//g" {} \; ## remove all """ (double quotation)
## find . -name \*.csv -exec sed -i "s/,\+/,/g" {} \; ## remove multiple occurrences of comma (,)
## find . -name \*.csv -exec sed -i "s/,$//" {} \; ## remove comma (,) at the end of each line
##
## find . -name \*.csv -exec sed -i "s/,//g" {} \; ## remove comma (,) at the end of each line: NO! This does something else. But what?

##############################################################################################
## design of a choice experiment 
##############################################################################################

library(support.CEs)  # package for survey construction 

## generate a balanced set of product profiles for survey
provider.survey <-
    Lma.design(attribute.names = 
               list(Brand = c("HP", "Toshiba", "Casper", "Sony"),
                    Processor = c("Intel Core i5", "Intel Core i7"),
                    Speed = c("1.8 GHz", "2.2 GHz", "2.4 GHz", "2.6 GHz"), ## GHz
                    Memory = c("4 GB","8 GB"), ## GB
                    HardDisk = c("500 GB", "750 GB"),
                    Screen = c("15.6 in", "17.3 in"),
                    Price = c("1500 TL", "1700 TL", "2000 TL", "3500 TL")), ## TL), ## inches
               nalternatives = 1, nblocks=1, row.renames = FALSE, seed=9999)
print(questionnaire(provider.survey))  # print survey design for review

sink("questions_for_survey.txt")  # send survey to external text file
questionnaire(provider.survey)
sink() # send output back to the screen

## extract survey form which is more suitable to write in a
## comma-separated file
(survey = provider.survey$alternatives$alt.1[,c(2,4:10)]) 
colnames(survey)[1] ="Alternative"

## add an empty rank columns for survey participant
(survey = data.frame(survey, RANK=rep("",nrow(survey)))) 

## write the survey to a comma-separated file after the instructions
## on how the survey has to be filled.
msg = sprintf("## Please rank %d laptops below\n## each with exactly one number\n## between 1 and %d\n## in the RANK column (last column):\n##  1 = most preferred,\n## %d = least preferred.\n## You may sort the rows\n## with respect to columns\n## if it helps.\n", nrow(survey), nrow(survey), nrow(survey))
cat(msg, file = "laptops.csv")
write.table(survey, "laptops.csv", sep =",", row.names = FALSE, append=TRUE)

##############################################################################################
## conjoint analysis of survey results
##############################################################################################

response = read.table("laptops-hande.csv", head=TRUE, sep=",")
head(response)
response = response[,-1] ## Remove Alternative column -- irrelevant
                         ## to the conjoint analysis
response[order(response$RANK),]
response$RANK = - response$RANK ## more natural for more attractive
                                ## features to have positive
                                ## part-worths

## (old.options = options('contrasts')$contrasts) ## store current the
                                             ## setting in order to
                                             ## get back to it after
                                             ## finishing the analysis

options(contrasts=c('contr.sum','contr.poly')) ## switch to sum
                                               ## contrasts before
                                               ## multip regression is
                                               ## fit
options('contrasts') ## double check
contr.sum(levels(response$Brand))

(res = lm(RANK ~ ., response)) ## calculate the part worths 
(coefs = coef(res)) ## printed in scientific format
round(coefs, digits=8) ## round to 8 digits for more readable format

partWorths = list()
categories = setdiff(names(response),"RANK")

for (i in seq_along(categories)){
    id = grep(paste(categories[i],".+",sep=""), names(coefs))    
    partWorths[categories[i]] = list(c(coefs[id],-sum(coefs[id])))
    names(partWorths[[categories[i]]]) = levels(response[[categories[i]]])
}

print(partWorths)

partImportance =
    lapply(partWorths, function(x) diff(range(x))) ## unnormalized
                                                   ## importance
                                                   ## measures
print(partImportance)
partImportance =
    sapply(partImportance, function(x) x/sum(unlist(partImportance))) ## part
                                                       ## importance
print(partImportance)
(sort(partImportance, decreasing=TRUE))
round(sort(partImportance, decreasing=TRUE), digits=8) ## more readable

barplot(partImportance)
## reorder partImportance and parthWorts in decreasing order of
## importance
ord = order(partImportance, decreasing=TRUE)
partImportance = partImportance[ord]
partWorths = partWorths[ord]

## plot part importance bar chart
par(mar=c(4,4,3,1))
bp = barplot(partImportance, main="Part Importance")
text(x=bp, y=partImportance,
     labels=sprintf("%d%%", round(100*partImportance)), pos=1)
dev.print(pdf, "partImportance.pdf")

## plot part worths
(pathWorths.array = unlist(partWorths))
round(pathWorths.array, digits=8)
part.numbers = sapply(partWorths, length)

par(mar=c(2,4,10,1))
plot(pathWorths.array, t="h", xaxt="n", ylab="Path worths",xlab="")
abline(h=0)
points(pathWorths.array)
axis(3, at = seq_along(pathWorths.array), label=names(pathWorths.array), las=2)
abline(v=cumsum(part.numbers)+0.5, lty="dashed")
dev.print(pdf, "partWorths.pdf")

## Calculate the utilities for the laptops from hepsiburada

dd = read.table("laptops-hepsiburada-hande.csv", header=TRUE, sep=",")
head(dd)
(RANK.pred = -predict(res, newdata=dd[,-c(1,9)])) ## estimate scores

RANK.pred[order(RANK.pred)] = seq_along(RANK.pred) ## rank the scores

(RANKS = data.frame(Alternative=dd$Alternative, RANK.obs = dd$RANK,
    RANK.pred = RANK.pred))

ord3 = order(RANK.pred)
par(mar=c(4,4,3,1), pty="s")
plot(RANK.obs[ord3] ~ RANK.pred[ord3], RANKS, type ="o",
     xlab="Predicted RANK", ylab="Observed RANK")
abline(0,1,col="gray")
dev.print(pdf, "predictedRANKS.pdf")

#####################################################################################
## Student surveys for market simulation
#####################################################################################
 
all.output.dir <- "conjoint"

dirname = "./student laptop ratings"
outfile.prefix = "all"

## dirname = "./student laptop ratings/Spring 2015"
## outfile.prefix = "S15"

## dirname = "./student laptop ratings/Spring 2014"
## outfile.prefix = "S14"

## dirname = "./student laptop ratings/Fall 2016"
## outfile.prefix = "F16"

## dirname = "./student laptop ratings/Spring 2017"
## outfile.prefix = "S17"

## dirname = "./student laptop ratings/Fall 2017"
## outfile.prefix = "F17"

output.dir <- file.path(all.output.dir, outfile.prefix)
dir.create(output.dir, recursive=TRUE)

survey.file.list = dir(dirname, pattern="[0-9]+.csv", recursive=TRUE)
head(survey.file.list)
##file.info(paste(dirname,survey.list[1], sep="/"))
require(tools)
survey.file.basenames =
    basename(file_path_sans_ext(paste(dirname,survey.file.list, sep="/")))
survey.file.dirnames =
    dirname(file_path_sans_ext(paste(dirname,survey.file.list, sep="/")))
head(survey.file.basenames)
head(survey.file.dirnames)

survey.list = list()

for (i in seq_along(survey.file.basenames)) {
    id = survey.file.basenames[i]
    directory = survey.file.dirnames[i]
    survey.train.file.name = paste(directory, "/",id,".csv",sep="")
    survey.test.file.name = paste(directory, "/",id, "-hepsiburada.csv",sep="")
    response.train = read.table(paste(survey.train.file.name,sep="/"),
                                header= T, sep=",")
    response.test = read.table(paste(survey.test.file.name,sep="/"),
                               header= T, sep=",")
    response.train = response.train[order(response.train$Alternative),]
    response.test = response.test[order(response.test$Alternative),]
    response.train.pred = data.frame(Alternative=response.train$Alternative,
                                     RANK = NA)
    response.test.pred = data.frame(Alternative=response.test$Alternative,
                                     RANK = NA)    
    survey.list = c(survey.list,
                    list(list(id=id, dir = directory,
                              response.train = response.train,
                              response.test = response.test,
                              response.train.pred =response.train.pred,
                              response.test.pred = response.test.pred)))    
}

str(survey.list)

## ## check if ranks are properly read into survey list
## require(plyr)
## require(magrittr)
## survey.response.train <- ldply(survey.list,
##                                .fun= function(x) data.frame(id=x$id, x$response.train))
## survey.response.train %>% dplyr::select(id, Alternative, RANK)
## survey.response.test <- ldply(survey.list,
##                                .fun= function(x) data.frame(id=x$id, x$response.test))
## survey.response.test %>% dplyr::select(id, Alternative, RANK)
## ## check is done

laptops.train.survey.results = data.frame()
laptops.test.survey.results = data.frame()
for (i in 1:length(survey.list)) {
    ## train
    person.survey = data.frame(id = survey.list[[i]]$id,
        survey.list[[i]]$response.train)
    laptops.train.survey.results = rbind(laptops.train.survey.results,
        person.survey)
    ## test
    person.survey = data.frame(id = survey.list[[i]]$id,
        survey.list[[i]]$response.test)
    laptops.test.survey.results = rbind(laptops.test.survey.results,
        person.survey)    
}

write.table(laptops.train.survey.results,
            sprintf("%s/laptops train survey results-%s.csv", output.dir, outfile.prefix),
            row.names = FALSE, sep =",")
write.table(laptops.test.survey.results,
            sprintf("%s/laptops test survey results-%s.csv", output.dir, outfile.prefix),
            row.names = FALSE, sep =",")
head(laptops.test.survey.results)

run.conjoint = function (response) {

    old.options = options('contrasts')$contrasts
    options(contrasts=c('contr.sum','contr.poly'))
    
    res = lm(-RANK ~ ., response) ## calculate the part worths 
    coefs = coef(res)

    partWorths = list()
    categories = setdiff(names(response),c("Alternative","RANK"))
    for (i in seq_along(categories)){
        id = grep(paste(categories[i],".+",sep=""), names(coefs))    
        partWorths[categories[i]] = list(c(coefs[id],-sum(coefs[id])))
        names(partWorths[[categories[i]]]) = levels(response[[categories[i]]])
    }    

    partImportance = lapply(partWorths, function(x) diff(range(x)))    
    partImportance = sapply(partImportance, function(x) x/sum(unlist(partImportance)))

    options(contrasts=old.options)
    return( list(res.lm = res, partWorths=partWorths, partImportance=partImportance))
}

## Apply conjoint analysis to every respondent of the survey
for (i in 1:length(survey.list)) {
    survey.list[[i]]$conjoint = run.conjoint(survey.list[[i]]$response.train)
    ## rank predictions on train set
    survey.list[[i]]$response.train.pred$RANK[
                        order(-predict(survey.list[[i]]$conjoint$res.lm))] =
        seq_along(survey.list[[i]]$response.train.pred$RANK)
    ## rank predictions on test set    
    survey.list[[i]]$response.test.pred$RANK[
                        order(-predict(survey.list[[i]]$conjoint$res.lm,
                                       newdata=survey.list[[i]]$response.test))] =
        seq_along(survey.list[[i]]$response.test.pred$RANK)        
}

##str(survey.list)

## 1(a) Plot part importance of each survey participant

pdf(sprintf("%s/partImportanceSurvey-%s.pdf", output.dir, outfile.prefix),
    paper="a4", width=0, height=0)
par(mfrow = c(5,2), mar=c(4,5,1,3))
for (i in 1:length(survey.list)) {
    partImportance = survey.list[[i]]$conjoint$partImportance
    bp = barplot(partImportance,
                 main=str_replace(survey.list[[i]]$id, "^([\\d]{2})\\d+(\\d)$", "\\1.....\\2"),
                 horiz=TRUE, yaxt="n")
    axis(2, at = bp, names(partImportance), las=1)
    text(y=bp, x=partImportance,
         labels=sprintf("%d%%", round(100*partImportance)), pos=2)       
}
dev.off()

## 1(c)i. Calculate the market share of each brand
marketshare.observed = rep(0, length(partWorths$Brand))
marketshare.simulated = rep(0, length(partWorths$Brand))

for (i in 1:length(survey.list)) {
    ## observed
    whoisthefirstchoice = which(survey.list[[i]]$response.test$RANK==1)
    firstBrand = survey.list[[i]]$response.test[whoisthefirstchoice,"Brand"]
    bpos = match(firstBrand, levels(firstBrand))
    marketshare.observed[bpos] = marketshare.observed[bpos] +1
    #simulated
    whoisthefirstchoice = which(survey.list[[i]]$response.test.pred$RANK==1)
    firstBrand = survey.list[[i]]$response.test[whoisthefirstchoice,"Brand"]
    bpos = match(firstBrand, levels(firstBrand))
    marketshare.simulated[bpos] = marketshare.simulated[bpos] +1
}
## observed
marketshare.observed = marketshare.observed / sum(marketshare.observed)
names(marketshare.observed) = levels(firstBrand)
print(marketshare.observed)
bp = barplot(marketshare.observed)
text(bp, marketshare.observed, labels=format(marketshare.observed,digits=1), pos=1)
dev.print(pdf, sprintf("%s/marketshare-observed-%s.pdf", output.dir, outfile.prefix))
## simulated
marketshare.simulated = marketshare.simulated / sum(marketshare.simulated)
names(marketshare.simulated) = levels(firstBrand)
print(marketshare.simulated)
bp = barplot(marketshare.simulated)
text(bp, marketshare.simulated, labels=format(marketshare.simulated,digits=1), pos=1)
dev.print(pdf, sprintf("%s/marketshare-simulated-%s.pdf", output.dir, outfile.prefix))

marketshare = rbind(marketshare.observed, marketshare.simulated)
rownames(marketshare) = c("observed", "simulated")
bp = barplot(marketshare, beside=TRUE, legend.text = TRUE,
             args.legend=list(x="topleft"),
             col=c("gray80", "gray95"))
text(bp, marketshare,
     labels=format(marketshare,digits=1), pos=1, cex=1)
dev.print(pdf, sprintf("%s/marketshare-%s.pdf", output.dir, outfile.prefix))

## Calculate the market share for each company's each model's market share
## observed
marketshare.observed.model = rep(0, nrow(survey.list[[1]]$response.test))
for (i in 1:length(survey.list)) {
    firstchoice = which(survey.list[[i]]$response.test$RANK==1)
    marketshare.observed.model[firstchoice] =
        marketshare.observed.model[firstchoice] + 1
}
marketshare.observed.model = marketshare.observed.model /
    sum(marketshare.observed.model)
names(marketshare.observed.model) = with(survey.list[[1]]$response.test,
                                paste(Brand, Alternative, sep="."))
bp = barplot(marketshare.observed.model)
text(bp, marketshare.observed.model,
     labels=format(marketshare.observed.model,digits=1), pos=1)
dev.print(pdf,
          sprintf("%s/marketshare-observed-ForEachModel-%s.pdf", output.dir, outfile.prefix))
## simulated
marketshare.simulated.model = rep(0, nrow(survey.list[[1]]$response.test))
for (i in 1:length(survey.list)) {
    firstchoice = which(survey.list[[i]]$response.test.pred$RANK==1)
    marketshare.simulated.model[firstchoice] =
        marketshare.simulated.model[firstchoice] + 1
}
marketshare.simulated.model = marketshare.simulated.model / sum(marketshare.simulated.model)
names(marketshare.simulated.model) = with(survey.list[[1]]$response.test,
                                paste(Brand, Alternative, sep="."))
bp = barplot(marketshare.simulated.model)
text(bp, marketshare.simulated.model,
     labels=format(marketshare.simulated.model,digits=1), pos=1)
dev.print(pdf,
          sprintf("%s/marketshare-simulated-ForEachModel-%s.pdf", output.dir, outfile.prefix))

marketshare.model = rbind(marketshare.observed.model, marketshare.simulated.model)
rownames(marketshare.model) = c("observed", "simulated")
bp = barplot(marketshare.model, beside=TRUE,
             legend.text=TRUE, args.legend = list(x="topright"),
             col=c("gray80","gray95"))
text(bp, marketshare.model,
     labels=format(marketshare.model,digits=1), pos=1, cex=0.8)
dev.print(pdf, sprintf("%s/marketshareForEachModel-%s.pdf", output.dir, outfile.prefix))


## 1(c)ii. calculate the substitution matrix

products = with(survey.list[[1]]$response.test,
                paste(Brand, Alternative, sep="."))
##observed                
substitution.observed.matrix = array(0, dim= rep(length(products),2),
                            dimnames= list(from = products, to=products))
for (i in 1:length(survey.list)) {
    id = order(survey.list[[i]]$response.test$RANK)
    from.id = head(id,-1)
    to.id = survey.list[[i]]$response.test$Alternative[tail(id,-1)]
    for (j in seq_along(from.id)) 
        substitution.observed.matrix[from.id[j],to.id[j]] =
            substitution.observed.matrix[from.id[j],to.id[j]] + 1
}
print(substitution.observed.matrix)
substitution.observed.rates.matrix = prop.table(substitution.observed.matrix,2)
##par(mfrow=c(ncol(substitution.observed.rates.matrix),1))
par(mfrow=c(3,2))
for (i in 1:ncol(substitution.observed.rates.matrix)){
    barplot(substitution.observed.rates.matrix[i,], horiz=TRUE, las=2,
            xlab ="observed substitution rate", ylab="",
            main =row.names(substitution.observed.rates.matrix)[i])
    mtext(side=2, line=5, text ="substitute  model")
}
par(mfrow=c(1,1))
dev.print(pdf, sprintf("%s/substitution-observed-%s.pdf", output.dir, outfile.prefix))
##simulated
substitution.simulated.matrix = array(0, dim= rep(length(products),2),
                            dimnames= list(from = products, to=products))
for (i in 1:length(survey.list)) {
    id = order(survey.list[[i]]$response.test.pred$RANK)
    from.id = head(id,-1)
    to.id = survey.list[[i]]$response.test$Alternative[tail(id,-1)]
    for (j in seq_along(from.id)) 
        substitution.simulated.matrix[from.id[j],to.id[j]] =
            substitution.simulated.matrix[from.id[j],to.id[j]] + 1
}
print(substitution.simulated.matrix)
substitution.simulated.rates.matrix = prop.table(substitution.simulated.matrix,2)
##par(mfrow=c(ncol(substitution.simulated.rates.matrix),1))
par(mfrow=c(3,2))
for (i in 1:ncol(substitution.simulated.rates.matrix)){
    barplot(substitution.simulated.rates.matrix[i,], horiz=TRUE, las=2,
            xlab ="simulated substitution rate", ylab="",
            main =row.names(substitution.simulated.rates.matrix)[i])
    mtext(side=2, line=5, text ="substitute  model")
}
par(mfrow=c(1,1))
dev.print(pdf, sprintf("%s/substitution-simulated-%s.pdf", output.dir, outfile.prefix))

par(mfrow=c(3,2))
for (i in 1:ncol(substitution.simulated.rates.matrix)){
    substitution.matrix = rbind(substitution.simulated.rates.matrix[i,],
                         substitution.observed.rates.matrix[i,])
    rownames(substitution.matrix)=c("simulated","observed")
    bp = barplot(substitution.matrix, horiz=TRUE, las=2,
                 beside=TRUE,
                 xlab ="substitution rate", ylab="",
                 main =row.names(substitution.simulated.rates.matrix)[i],
                 col=c("gray95","gray80"))
    text(substitution.matrix,bp,round(substitution.matrix,2), pos=2, cex=0.6)
    mtext(side=2, line=5, text ="substitute  model")
}
par(mfrow=c(1,1))
dev.print(pdf, sprintf("%s/substitution-%s.pdf", output.dir, outfile.prefix))

## 1(c)iii.

## start with the existing market
market = survey.list[[1]]$response.test
market$RANK = NA

## create alternative products to replace an existing product (if the
## Alternative no exist)

## Create new.product.alternatives
new.product.alternatives = c()
old.product = survey.list[[1]]$response.test[4,]
old.product$RANK = NA
feature.names = setdiff(names(old.product), c("Alternative","RANK"))
for (i in 1:length(feature.names)){
    if (feature.names[i] != "Brand") {
        fields = setdiff(names(partWorths[[feature.names[i]]]),
                         old.product[[feature.names[i]]])
        ncat = length(fields)
        new.products = old.product[rep(1, ncat),]
        new.products[[feature.names[i]]] = fields
        new.product.alternatives = rbind(new.product.alternatives,
                                         new.products)
    }
}

marketshare.newproducts =
    array(0, dim=c(nrow(new.product.alternatives), nrow(market)),
          dimnames = list(product.no=1:nrow(new.product.alternatives),
                          model=paste(market$Brand, market$Alternative, sep=".")))
                          
for (i in 1:nrow(new.product.alternatives)){
    new.market = market
    id = match(new.product.alternatives$Alternative[i], new.market$Alternative)
    new.market[id,] = new.product.alternatives[i,]
    for (j in 1:length(survey.list)) {        
        firstchoice = which.min(-predict(survey.list[[j]]$conjoint$res.lm,
                                         newdata=new.market))
        marketshare.newproducts[i,firstchoice] =
            marketshare.newproducts[i,firstchoice] + 1        
    }
}

marketshare.newproducts = prop.table(marketshare.newproducts,1)
marketshare.simulated.model

plot(marketshare.newproducts[,"Casper.4"], t="h",
     ylab="Market share of redesigned Casper 4", xlab = "Product No.",
     las=1, xaxt="n")
axis(1, at=seq_along(marketshare.newproducts[,"Casper.4"]),
     labels=seq_along(marketshare.newproducts[,"Casper.4"]))
## text(seq_along(marketshare.newproducts[,"Casper.4"]),
##      marketshare.newproducts[,"Casper.4"], pos=3,
##      label=round(marketshare.newproducts[,"Casper.4"],2))
abline(h=c(0, marketshare.simulated.model["Casper.4"]),
       lty=c("solid", "dashed"))
mtext(side=2, line=1, at=marketshare.simulated.model["Casper.4"],
      text = round(marketshare.simulated.model["Casper.4"],2), las=2)
legend("topright", legend="current market share", lty="dashed", bty="n")
feature.text = c()
for (i in 1:nrow(marketshare.newproducts)) {
    feature = new.product.alternatives[i,] != old.product
    feature[1,c("Alternative","RANK")] = FALSE
    feature.text = c(feature.text,
                     paste(round(marketshare.newproducts[i,"Casper.4"],2), "\n",
                           names(old.product)[feature], "\n",
                           new.product.alternatives[i,feature],
                           "\n(", old.product[1,feature], ")", sep=""))
}
text(seq_along(marketshare.newproducts[,"Casper.4"]),
     marketshare.newproducts[,"Casper.4"],
     labels=feature.text, cex=0.8,
     pos=c(1, rep(3,9)))
dev.print(pdf, file=sprintf("%s/new-product-%s.pdf", output.dir, outfile.prefix))

id = which.max(marketshare.newproducts [,"Casper.4"])

marketshare.np = rbind(oldprod =marketshare.simulated.model,
    newprod = marketshare.newproducts[id,])

bp = barplot(marketshare.np, beside=TRUE,
             legend.text=c("market with old product",
                           "market with new product"),
             args.legend = list(x="topright"),
             col=c("gray80","gray95"))
text(bp, marketshare.np,
     labels=format(marketshare.np,digits=1), pos=1, cex=0.8)

dev.print(pdf, sprintf("%s/marketshare-best-new-and-old-products-%s.pdf",
                       output.dir, outfile.prefix))

